/*	$OpenBSD: crypto_arch.h,v 1.6 2026/01/17 16:18:31 jsing Exp $ */
/*
 * Copyright (c) 2024 Joel Sing <jsing@openbsd.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdint.h>

#ifndef HEADER_CRYPTO_ARCH_H
#define HEADER_CRYPTO_ARCH_H

#define HAVE_CRYPTO_CPU_CAPS_INIT

#ifndef __ASSEMBLER__
extern uint64_t crypto_cpu_caps_aarch64;
#endif

#define CRYPTO_CPU_CAPS_AARCH64_AES	(1ULL << 0)
#define CRYPTO_CPU_CAPS_AARCH64_PMULL	(1ULL << 1)
#define CRYPTO_CPU_CAPS_AARCH64_SHA1	(1ULL << 2)
#define CRYPTO_CPU_CAPS_AARCH64_SHA2	(1ULL << 3)
#define CRYPTO_CPU_CAPS_AARCH64_SHA512	(1ULL << 4)
#define CRYPTO_CPU_CAPS_AARCH64_SHA3	(1ULL << 5)

#ifndef OPENSSL_NO_ASM

#ifdef LIBRESSL_USE_SHA_ASSEMBLY
#define HAVE_SHA1_BLOCK_DATA_ORDER
#define HAVE_SHA256_BLOCK_DATA_ORDER
#define HAVE_SHA512_BLOCK_DATA_ORDER
#endif

#endif

#endif
