/*	$OpenBSD: crypto_arch.h,v 1.3 2026/01/17 16:18:32 jsing Exp $ */
/*
 * Copyright (c) 2024 Joel Sing <jsing@openbsd.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef HEADER_CRYPTO_ARCH_H
#define HEADER_CRYPTO_ARCH_H

#ifndef OPENSSL_NO_ASM

#ifdef LIBRESSL_USE_AES_ASSEMBLY
#define HAVE_AES_SET_ENCRYPT_KEY_INTERNAL
#define HAVE_AES_SET_DECRYPT_KEY_INTERNAL
#define HAVE_AES_ENCRYPT_INTERNAL
#define HAVE_AES_DECRYPT_INTERNAL
#endif

#ifdef LIBRESSL_USE_SHA_ASSEMBLY
#define HAVE_SHA1_BLOCK_DATA_ORDER
#define HAVE_SHA1_BLOCK_GENERIC
#define HAVE_SHA256_BLOCK_DATA_ORDER
#define HAVE_SHA256_BLOCK_GENERIC
#define HAVE_SHA512_BLOCK_DATA_ORDER
#define HAVE_SHA512_BLOCK_GENERIC
#endif

#endif

#endif
